package de.ozgcloud.bescheid.vorgang;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.bescheid.vorgang.FormDataEntry.FormField;
import de.ozgcloud.bescheid.vorgang.FormDataEntry.SubForm;
import de.ozgcloud.bescheid.vorgang.Vorgang.Eingang;
import lombok.SneakyThrows;

class FormDataEntrySerializerTest {

	private ObjectMapper mapper = new ObjectMapper();

	@Test
	void schouldSerializeFormDataWithSingleField() throws JsonProcessingException {
		var eingang = Eingang.builder()
				.formData(FormField.builder().name("name").value("Thea").build())
				.build();

		var json = mapper.writeValueAsString(eingang);

		assertThat(json).contains("\"formData\":{\"name\":\"Thea\"}");
	}

	@Test
	@SneakyThrows
	void sohludSerializeFormDataWithSubForm() {
		var eingang = Eingang.builder()
				.formData(SubForm.builder().name("address")
						.entry(FormField.builder().name("name").value("Thea").build())
						.build())
				.build();

		var json = mapper.writeValueAsString(eingang);

		assertThat(json).contains("\"formData\":{\"address\":{\"name\":\"Thea\"}}");
	}

	@Test
	@SneakyThrows
	void shouldSerializeFormDataWithSubSubForm() {
		var eingang = Eingang.builder()
				.formData(SubForm.builder().name("address")
						.entry(SubForm.builder().name("person")
								.entry(FormField.builder().name("name").value("Thea").build())
								.build())
						.build())
				.build();

		var json = mapper.writeValueAsString(eingang);

		assertThat(json).contains("\"formData\":{\"address\":{\"person\":{\"name\":\"Thea\"}}}");
	}

	@Test
	@SneakyThrows
	void shouldSerializeFormWithTwoFields() {
		var eingang = Eingang.builder()
				.formData(FormField.builder().name("firstname").value("Thea").build())
				.formData(FormField.builder().name("lastname").value("Test").build())
				.build();

		var json = mapper.writeValueAsString(eingang);

		assertThat(json).contains("\"formData\":{\"firstname\":\"Thea\",\"lastname\":\"Test\"}");
	}
}
