package de.ozgcloud.vorgang.common.migration;

import org.springframework.data.mongodb.core.MongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;

@ChangeUnit(id = "2024-04-02 17:15:00 OZG-4770", order = "M009", author = "ebardin", runAlways = true)
public class M009_SetServiceKontoTrustLevel { // NOSONAR

	private static final String VORGANG_COLLECTION = "vorgang";
	private static final String DEFAULT_TRUST_LEVEL = "STORK-QAA-Level-1";
	private static final String KEY_SERVICE_KONTO = "header.serviceKonto";
	private static final String KEY_TRUST_LEVEL = KEY_SERVICE_KONTO + ".trustLevel";

	@Execution
	public void doMigration(MongoOperations db) {
		db.updateMulti(createFilterQuery(), createUpdate(), VORGANG_COLLECTION);
	}

	Query createFilterQuery() {
		var serviceKontoExistsCriteria = Criteria.where(KEY_SERVICE_KONTO).exists(true);
		var trustLevelNotExistsCriteria = Criteria.where(KEY_TRUST_LEVEL).exists(false);
		return new Query(new Criteria().andOperator(serviceKontoExistsCriteria, trustLevelNotExistsCriteria));
	}

	Update createUpdate() {
		return new Update().set(KEY_TRUST_LEVEL, DEFAULT_TRUST_LEVEL);
	}

	@RollbackExecution
	public void rollback() {
		// kein rollback implementiert
	}
}