package de.ozgcloud.vorgang.common.migration;

import static de.ozgcloud.vorgang.common.migration.M005_UpdateNachrichtenManagerClientNameInClientAttributes.*;
import static org.assertj.core.api.Assertions.*;

import org.bson.Document;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Query;

import de.ozgcloud.common.test.DataITCase;

@DataITCase
class M005_UpdateNachrichtenManagerClientNameInClientAttributesITCase {

	private final static Document OLD_CLIENT_NAME_VORGANG = RenameTestFactory.createVorgangWithClientAttributes(OLD_CLIENT_NAME);

	private final M005_UpdateNachrichtenManagerClientNameInClientAttributes migration = new M005_UpdateNachrichtenManagerClientNameInClientAttributes();

	@Autowired
	private MigrationDbTestUtils dbTestUtils;
	@Autowired
	private MongoTemplate template;

	@BeforeEach
	public void init() {
		dbTestUtils.dropVorgangCollection();
	}

	@DisplayName("Do migration")
	@Nested
	class TestDoMigration {

		@BeforeEach
		public void init() {
			dbTestUtils.saveVorgang(OLD_CLIENT_NAME_VORGANG);
		}

		@Test
		void shouldUpdateClientName() {
			migration.doMigration(template);

			var vorgaenge = dbTestUtils.findVorgang(new Query());

			assertThat(vorgaenge).hasSize(1);
			var vorgangClientAttributes = (Document) vorgaenge.get(0).get(RenameUtil.CLIENT_ATTRIBUTES_KEY);
			assertThat(vorgangClientAttributes).hasSize(1).containsKey(NEW_CLIENT_NAME);
		}

		@Test
		void shouldKeeptClientAttributesData() {
			migration.doMigration(template);

			var vorgaenge = dbTestUtils.findVorgang(new Query());

			assertThat(vorgaenge).hasSize(1);
			var clientAttribute = getClientAttribute(vorgaenge.get(0), RenameTestFactory.DUMMY_ATTRIBUTE_NAME);
			assertThat(clientAttribute).containsExactly(
					entry(RenameTestFactory.CLIENT_ATTRIBUTE_ACCESS_KEY, RenameTestFactory.CLIENT_ATTRIBUTE_ACCESS_VALUE),
					entry(RenameTestFactory.CLIENT_ATTRIBUTE_VALUE_KEY, RenameTestFactory.CLIENT_ATTRIBUTE_VALUE_VALUE),
					entry(RenameTestFactory.CLIENT_ATTRIBUTE_CLASS_KEY, RenameTestFactory.CLIENT_ATTRIBUTE_CLASS_VALUE));
		}

		private Document getClientAttribute(Document vorgang, String name) {
			var vorgangClientAttributes = (Document) vorgang.get(RenameUtil.CLIENT_ATTRIBUTES_KEY);
			var clientAttributeByClientName = (Document) vorgangClientAttributes.get(NEW_CLIENT_NAME);
			return (Document) clientAttributeByClientName.get(name);
		}
	}
}